TAG_DATETIME = 0
TAG_NONE = 6
TAG_TABLE_NAME = 7
TAG_RECORD_ID = 8
TAG_UUID_STRING = 9
TAG_DECIMAL_STRING = 10

TAG_RANGE = 49
TAG_BOUND_INCLUDED = 50
TAG_BOUND_EXCLUDED = 51

TAG_DATETIME_COMPACT = 12
TAG_DURATION = 13
TAG_DURATION_COMPACT = 14

TAG_GEOMETRY_POINT = 88
TAG_GEOMETRY_LINE = 89
TAG_GEOMETRY_POLYGON = 90
TAG_GEOMETRY_MULTI_POINT = 91
TAG_GEOMETRY_MULTI_LINE = 92
TAG_GEOMETRY_MULTI_POLYGON = 93
TAG_GEOMETRY_COLLECTION = 94
